@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -115,6 +118,42 @@ class AssetServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return AssetServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> AssetServiceTransport:
         """Returns the transport used by the client instance.
@@ -197,6 +236,34 @@ class AssetServiceAsyncClient:
         the export operation result. For regular-size resource parent,
         the export operation usually finishes within 5 minutes.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_export_assets():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                output_config = asset_v1.OutputConfig()
+                output_config.gcs_destination.uri = "uri_value"
+
+                request = asset_v1.ExportAssetsRequest(
+                    parent="parent_value",
+                    output_config=output_config,
+                )
+
+                # Make the request
+                operation = client.export_assets(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.ExportAssetsRequest, dict]):
                 The request object. Export asset request.
@@ -260,6 +327,27 @@ class AssetServiceAsyncClient:
         r"""Lists assets with time and resource types and returns
         paged results in response.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_list_assets():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.ListAssetsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_assets(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.ListAssetsRequest, dict]):
                 The request object. ListAssets request.
@@ -288,7 +376,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -356,6 +444,26 @@ class AssetServiceAsyncClient:
         specified asset does not exist, this API returns an
         INVALID_ARGUMENT error.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_batch_get_assets_history():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.BatchGetAssetsHistoryRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.batch_get_assets_history(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.BatchGetAssetsHistoryRequest, dict]):
                 The request object. Batch get assets history request.
@@ -415,6 +523,31 @@ class AssetServiceAsyncClient:
         project/folder/organization to listen to its asset
         updates.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_create_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                feed = asset_v1.Feed()
+                feed.name = "name_value"
+
+                request = asset_v1.CreateFeedRequest(
+                    parent="parent_value",
+                    feed_id="feed_id_value",
+                    feed=feed,
+                )
+
+                # Make the request
+                response = client.create_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.CreateFeedRequest, dict]):
                 The request object. Create asset feed request.
@@ -450,7 +583,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -497,6 +630,25 @@ class AssetServiceAsyncClient:
     ) -> asset_service.Feed:
         r"""Gets details about an asset feed.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_get_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.GetFeedRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.GetFeedRequest, dict]):
                 The request object. Get asset feed request.
@@ -527,7 +679,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -585,6 +737,26 @@ class AssetServiceAsyncClient:
         r"""Lists all asset feeds in a parent
         project/folder/organization.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_list_feeds():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.ListFeedsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.list_feeds(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.ListFeedsRequest, dict]):
                 The request object. List asset feeds request.
@@ -610,7 +782,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -667,6 +839,28 @@ class AssetServiceAsyncClient:
     ) -> asset_service.Feed:
         r"""Updates an asset feed configuration.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_update_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                feed = asset_v1.Feed()
+                feed.name = "name_value"
+
+                request = asset_v1.UpdateFeedRequest(
+                    feed=feed,
+                )
+
+                # Make the request
+                response = client.update_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.UpdateFeedRequest, dict]):
                 The request object. Update asset feed request.
@@ -698,7 +892,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([feed])
         if request is not None and has_flattened_params:
@@ -747,6 +941,22 @@ class AssetServiceAsyncClient:
     ) -> None:
         r"""Deletes an asset feed.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_delete_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.DeleteFeedRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_feed(request=request)
+
         Args:
             request (Union[google.cloud.asset_v1.types.DeleteFeedRequest, dict]):
                 The request object.
@@ -766,7 +976,7 @@ class AssetServiceAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -827,6 +1037,27 @@ class AssetServiceAsyncClient:
         the ``cloudasset.assets.searchAllResources`` permission on the
         desired scope, otherwise the request will be rejected.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_search_all_resources():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.SearchAllResourcesRequest(
+                    scope="scope_value",
+                )
+
+                # Make the request
+                page_result = client.search_all_resources(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.SearchAllResourcesRequest, dict]):
                 The request object. Search all resources request.
@@ -938,7 +1169,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([scope, query, asset_types])
         if request is not None and has_flattened_params:
@@ -1008,6 +1239,27 @@ class AssetServiceAsyncClient:
         ``cloudasset.assets.searchAllIamPolicies`` permission on the
         desired scope, otherwise the request will be rejected.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_search_all_iam_policies():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.SearchAllIamPoliciesRequest(
+                    scope="scope_value",
+                )
+
+                # Make the request
+                page_result = client.search_all_iam_policies(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.SearchAllIamPoliciesRequest, dict]):
                 The request object. Search all IAM policies request.
@@ -1101,7 +1353,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([scope, query])
         if request is not None and has_flattened_params:
@@ -1165,6 +1417,29 @@ class AssetServiceAsyncClient:
         r"""Analyzes IAM policies to answer which identities have
         what accesses on which resources.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_analyze_iam_policy():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                analysis_query = asset_v1.IamPolicyAnalysisQuery()
+                analysis_query.scope = "scope_value"
+
+                request = asset_v1.AnalyzeIamPolicyRequest(
+                    analysis_query=analysis_query,
+                )
+
+                # Make the request
+                response = client.analyze_iam_policy(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.AnalyzeIamPolicyRequest, dict]):
                 The request object. A request message for
@@ -1236,6 +1511,37 @@ class AssetServiceAsyncClient:
         to poll the operation result. The metadata contains the metadata
         for the long-running operation.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_analyze_iam_policy_longrunning():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                analysis_query = asset_v1.IamPolicyAnalysisQuery()
+                analysis_query.scope = "scope_value"
+
+                output_config = asset_v1.IamPolicyAnalysisOutputConfig()
+                output_config.gcs_destination.uri = "uri_value"
+
+                request = asset_v1.AnalyzeIamPolicyLongrunningRequest(
+                    analysis_query=analysis_query,
+                    output_config=output_config,
+                )
+
+                # Make the request
+                operation = client.analyze_iam_policy_longrunning(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.AnalyzeIamPolicyLongrunningRequest, dict]):
                 The request object. A request message for
@@ -1305,6 +1611,27 @@ class AssetServiceAsyncClient:
         subject to change before the actual resource migration
         takes place.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_analyze_move():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.AnalyzeMoveRequest(
+                    resource="resource_value",
+                    destination_parent="destination_parent_value",
+                )
+
+                # Make the request
+                response = client.analyze_move(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.AnalyzeMoveRequest, dict]):
                 The request object. The request message for performing
