@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -52,8 +52,8 @@ class StandardResourceMetadata(proto.Message):
             attributes is subject to change. For example:
             project id, DNS name etc.
         location (str):
-            Location can be "global", regional like "us-
-            ast1", or zonal like "us-west1-b".
+            Location can be "global", regional like
+            "us-east1", or zonal like "us-west1-b".
         labels (Sequence[google.cloud.asset_v1p1beta1.types.StandardResourceMetadata.LabelsEntry]):
             Labels associated with this resource. See `Labelling and
             grouping GCP
